package com.codigoparallevar.minicards.bridge.blocks;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.content.Context;
import android.os.Build;
import android.os.VibrationEffect;
import android.os.Vibrator;

import androidx.core.app.NotificationCompat;

import com.codigoparallevar.minicards.R;
import com.codigoparallevar.minicards.bridge.ProgramakerBridgeService;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

public class DefaultAndroidBlocks {

    private static long VIBRATION_REST_TIME = 1000; // Ms
    private static long VIBRATION_ACTIVE_TIME = 500; // Ms

    // This means that will take at most ~15 seconds
    private static int MAX_VIBRATION_CYCLES = 10;

    public static BridgeBlockListBuilder GetBuilder(Context ctx) {
        // System services
        NotificationManager notificationManager = (NotificationManager) ctx.getSystemService(Context.NOTIFICATION_SERVICE);
        Vibrator vibrator = (Vibrator) ctx.getSystemService(Context.VIBRATOR_SERVICE);

        assert notificationManager != null;
        assert vibrator != null;

        Random notificationRandom = new Random();

        if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.O) {
            NotificationChannel channel = new NotificationChannel(
                    ProgramakerBridgeService.BridgeUserNotificationChannel,
                    ProgramakerBridgeService.BridgeUserNotificationChannelName,
                    NotificationManager.IMPORTANCE_DEFAULT);
            notificationManager.createNotificationChannel(channel);

            NotificationChannel vibrationChannel = new NotificationChannel(
                    ProgramakerBridgeService.BridgeUserVibrationNotificationChannel,
                    ProgramakerBridgeService.BridgeUserVibrationNotificationChannelName,
                    NotificationManager.IMPORTANCE_DEFAULT);

            // Vibration is manually handled for Android O (or superior) devices
            vibrationChannel.setVibrationPattern(null);
            notificationManager.createNotificationChannel(vibrationChannel);
        }


        return new BridgeBlockListBuilder()
                // Notifications
                .addOperation(
                        "notifications_new",
                        "Create notification. Title: %1, text: %2",
                        new LinkedList<BlockArgumentDefinition>() {{
                            add(new BlockArgumentDefinition(BlockArgumentDefinition.Type.STRING, "My notification"));
                            add(new BlockArgumentDefinition(BlockArgumentDefinition.Type.STRING, "Sample description"));
                        }},
                        (List<?> params) -> {
                            if (params.size() != 2) {
                                throw new Exception("Expected two (2) arguments, found " + params.size());
                            }
                            String title = params.get(0).toString();
                            String description = params.get(1).toString();

                            Notification notif = new NotificationCompat
                                    .Builder(ctx, ProgramakerBridgeService.BridgeUserNotificationChannel)
                                    .setContentTitle(title)
                                    .setContentText(description)
                                    .setSmallIcon(R.drawable.ic_center_focus_weak_black) // TODO: Change icon
                                    .setPriority(NotificationCompat.PRIORITY_DEFAULT)
                                    .build();

                            int notificationId = notificationRandom.nextInt();
                            notificationManager.notify(notificationId, notif);
                        }
                )
                .addOperation(
                        "notifications_clear",
                        "Clear notifications",
                        Collections.emptyList(),
                        (List<?> params) -> {
                            notificationManager.cancelAll();
                        }
                )
                .addOperation(
                        "vibration_start",
                        "Vibrate %1 times",
                        new LinkedList<BlockArgumentDefinition>() {{
                            add(new BlockArgumentDefinition(BlockArgumentDefinition.Type.INT, "3"));
                        }},
                        (List<?> params) -> {
                            if (params.size() != 1) {
                                throw new Exception("Expected one (1) argument1, found " + params.size());
                            }

                            int userTimes = Integer.parseInt(params.get(0).toString());
                            final int times = Math.max(1, Math.min(MAX_VIBRATION_CYCLES, userTimes));

                            long[] pattern = new long[times * 2];
                            for (int i = 0; i < times; i++) {
                                pattern[i * 2] = VIBRATION_ACTIVE_TIME;
                                pattern[i * 2 + 1] = VIBRATION_REST_TIME;
                            }
                            // pattern[0] = 0; // Start immediately

                            Notification notif = new NotificationCompat
                                    .Builder(ctx, ProgramakerBridgeService.BridgeUserVibrationNotificationChannel)
                                    .setContentTitle(ctx.getString(R.string.vibration_activated))
                                    .setSmallIcon(R.drawable.ic_center_focus_weak_black) // TODO: Change icon
                                    .setPriority(NotificationCompat.PRIORITY_DEFAULT)
                                    .setAutoCancel(true)
                                    .setVibrate(pattern)
                                    .build();

                            int notificationId = notificationRandom.nextInt();
                            notificationManager.notify(notificationId, notif);

                            if (android.os.Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                                // Manually handle vibration if notification channels are in use
                                vibrator.vibrate(VibrationEffect.createWaveform(pattern, -1));
                            }

                            new Thread( () -> {
                                // Wait the activation time
                                for (int i = 0; i < times; i++) {
                                    try {
                                        Thread.sleep(pattern[i * 2]);
                                        Thread.sleep(pattern[i * 2 + 1]);
                                    } catch (InterruptedException e) {
                                        e.printStackTrace();
                                    }
                                }

                                // Remove notification
                                notificationManager.cancel(notificationId);
                            }).start();
                        }
                )
                ;
    }
}
